// pfind.c 
//


#include                "pfind.h"

int main(int ac, char **av)
{
    int    i;
    char   mode      = 'd',        // mode: 'd' = get start directory
                                   //       'f' = get file name
           *startDir = NULL,
           *fileName = NULL;
    
    // Finite state machine for reading modes and arguments
    for ( i = 1; i < ac; i++ ) {
        
            // mode change?
            if (av[i][0] == '-') {
                    
                    // mode change
                    switch (av[i][1]) {
                        case 'd': case 'f':
                            mode = av[i][1];
                            break;
                        default:
                            fatal("Unknown option");
                    }
            }
            else {
                    // read argument
                    switch (mode) {
                        case 'd':
                            startDir = av[i];
                            mode = 'f';                   // mode change
                            break;
                        case 'f':
                            fileName = av[i];
                            break;
                    }
            }
    }
    
    if (! startDir)                         
            fatal("Directory(-d) needs to be specified");
 
    if (! fileName)                         
            fatal("File name(-f) needs to be specified");
    
    searchdir( startDir, fileName );
    
    return (0);
}


//
//  search directory dirName for fileName and print
//
void searchdir( char *dirName, char *fileName )
{
    DIR           * dir_ptr;            // the directory
    struct dirent * direntp;            // each entry
    char          * pathName;           // store pathname
    
    
    // bad directory?
    if (( dir_ptr = opendir( dirName )) == NULL ){
            fprintf(stderr,"find: ");
            perror( dirName );
    }
    
    // '.' fileName
    else if ( strcmp(fileName, "." ) == 0 )
            printf( ".\n" );
    
    // '..' filename
    else if ( strcmp(fileName, "..") == 0 )
            ;
                
    else {
            while (( direntp = readdir( dir_ptr )) != NULL ) {
                    
	            // get memory for pathname string
	            pathName = malloc( 1 + strlen( dirName) + 
                                           strlen( direntp->d_name));
	            if (pathName == NULL)    // out of memory
		        fatal("Out of memory");
                    
                    // get path name
                    sprintf( pathName, "%s/%s", dirName, direntp->d_name );

                    // if filename matches, print
                    if ( strcmp(direntp->d_name, fileName) == 0 )
                            printf("%s\n", pathName );

                    // ignore '.' and '..' directories
                    if ( strcmp(direntp->d_name, "." ) != 0 &&
                         strcmp(direntp->d_name, "..") != 0)
                            
                            // if a directory, recursivly drill through...
                            if ( checkIfDir ( pathName ) )
                                     searchdir( pathName, fileName );

                    free(pathName);
            }
            closedir(dir_ptr);
    }
}


//
//  is pathName a directory?
//
int checkIfDir( char *pathName )
{
        struct stat info;
        
        if ( lstat(pathName, &info) == -1 )    // cannot stat
            perror( pathName );
        
        // use S_ISREG(m), check for regular file
        // use S_ISLNK(m), check for symbolic link
        return(S_ISDIR(info.st_mode));
}
