////////////
//
//  sttyl.c 
//
//      AUTHOR: Russell Lowke
//   CSCIE-215: Assignment Three
//
//
//       Usage: sttyl [-]icrnl        // map CR to NL (ala CRMOD)
//                    [-]onlcr        // map NL to CR-NL (ala CRMOD)
//                    [-]echo         // enable echoing
//                    [-]echoe        // visually erase chars
//                    [-]olcuc        // map lowercase to uppercase on output
//                    [-]tabs         // expand tabs on output
//                    [-]icano        // canonicalize input lines
//                    [-]icanon       // canonicalize input lines
//                    [-]isig         // enable signals INTR, QUIT, [D]SUSP
//                    [-]hupcl        // hang up on last close
//                    [-]cread        // enable receiver
//                    [-]brkint       // map BREAK to SIGINTR
//                    [-]inpck        // disable checking of parity errors
//                    [-]iexten       // enable FLUSHO and LNEXT
//
//                    intr  CHAR      // CHAR will send an interrupt signal
//                    erase CHAR      // CHAR will erase last character typed
//                    kill  CHAR      // CHAR will erase the current line
//                    start CHAR      // CHAR will restart output after stopping
//                    stop  CHAR      // CHAR will stop the output
//
//
// Description: Mimics the basic functionality of the Unix stty,
//              sttyl is used to print or change terminal characteristics.
//
//      Method: main reads args, ...
//
//       Notes: sttyl only works properly in sh
//              CHAR setting for backspace is ^?
//              CHAR commands are best passed in quotes, ie  "^?"
//


// include header
#include                "sttyl.h"


//
//  mFlagArgs    
//
//     struct of type flagTable
//     Holds the attribute, value, flag type and description,
//     for every argument flag accepted by sttyl
//
struct flagTable mFlagArgs[] = {
    {"icrnl" , ICRNL , INPUTFLAG  , "map CR to NL (ala CRMOD)"             },
    {"onlcr" , ONLCR , OUTPUTFLAG , "map NL to CR-NL (ala CRMOD)"          },
    {"echo"  , ECHO  , LOCALFLAG  , "enable echoing"                       },
    {"echoe" , ECHOE , LOCALFLAG  , "visually erase chars"                 },
    {"olcuc" , OLCUC , OUTPUTFLAG , "map lowercase to uppercase on output" },
    {"tabs"  , TAB3  , OUTPUTFLAG , "expand tabs on output"                },
    {"icano" , ICANON, LOCALFLAG  , "canonicalize input lines"             },
    {"icanon", ICANON, LOCALFLAG  , "canonicalize input lines"             },
    {"isig"  , ISIG  , LOCALFLAG  , "enable signals INTR, QUIT, [D]SUSP"   },
    {"hupcl" , HUPCL , CONTROLFLAG, "hang up on last close"                },
    {"cread" , CREAD , CONTROLFLAG, "enable receiver"                      },
    {"brkint", BRKINT, INPUTFLAG  , "map BREAK to SIGINTR"                 },
    {"inpck" , INPCK , INPUTFLAG  , "disable checking of parity errors"    },
    {"iexten", IEXTEN, LOCALFLAG  , "enable FLUSHO and LNEXT"              },
    { NULL   , 0     , 0          ,  NULL                                  }};


//
//  mKeyArgs
//
//     struct of type keyTable
//     Holds the attribute and value,
//     for every modifable key argument accepted by sttyl 
//
struct keyTable mKeyArgs[] = {
    {"intr"  , VINTR },
    {"erase" , VERASE},
    {"kill"  , VKILL },
    {"start" , VSTART},
    {"stop"  , VSTOP },
    { NULL   , 0     }};


///////
//
//  main
//
//     loop through arguments passed into main via ac and av
//     starts at av[1] as av[0] is the program name, in this case "sttyl"

    // convert all 'my'  to 'm' or remove 'my'
    
int main(int ac, char **av)
{
    struct termios info;                     // struct to hold termios info
    int    i,                                // general purpose counter
           switchIs,                         // indicates if flag is,
                                             // turning OFF (0)  or ON (1)
           item;                             // position of item in flagTable,
                                             //                  or keyTable
    char   ascii,                            // modified ascii of CHAR key
          *argName;                          // pointer to argument name
    
        
    if (tcgetattr(0, &info) < 0)             // load terminal arributes to info
        oops("Couldn't get termios");        // ...error
    

    for ( i = 1; i < ac; i++ ) {             // loop through args given to main
        
        ascii    =  0;                       // cleanup ascii and item vars
        
        switchIs = (av[i][0] == '-');        // switchIs to 0, or 1 
                                             //   depending on '-' operator
        
        argName  = &av[i][switchIs];         // pointer to argument name,
                                             //   excluding '-'
        
                                             // if argName has a flagIndex
        if ((item = flagIndex(argName)) != -1)
            setFlag(&info, item, switchIs);  // set flag [item] to [switchIs]
        
                                             // if argName has a keyIndex
                                             // and there is another argument
        else if ((item = keyIndex(argName)) != -1 
                    && i+1 < ac ) {
            
            ascii = getAscii(av[++i]);       // increment i and get ascii

            
            if (ascii)                       // if valid ascii
                info.c_cc[mKeyArgs[item].theValue] = ascii;   // set char key
            else
                fatal("Invalid argument");   // ...error
        } 
        else
            fatal("Unknown mode");           // ...error
    }
    
    tcsetattr(0, TCSANOW, &info);

    if (ac == 1) 
        displaySettings(&info);
    
    return (0);
}



///////
//
//  flagIndex
//
//     Search directory [dirName] looking for [fileName]. Matches are
//     printed using printf.  When a directory is found searchdir calls
//     itself passing that directory as the dirName. 
//
//     args:  myString     --  path of directory to search.
//
int flagIndex(char *myString)
{
    int i;
    
    for ( i = 0; mFlagArgs[i].theValue ; i++ ) {
        if ( strcmp(mFlagArgs[i].attribute, myString) == 0 )
            return (i);
    }

    return -1;
}


///////
//
//  keyIndex
//
//     Search directory [dirName] looking for [fileName]. Matches are
//     printed using printf.  When a directory is found searchdir calls
//     itself passing that directory as the dirName. 
//
//     args:  myString     --  path of directory to search.
//
int keyIndex(char *myString)
{
    int i;
    
    for ( i = 0; mKeyArgs[i].theValue ; i++ ) {
        if ( strcmp(mKeyArgs[i].attribute, myString) == 0 )
            return (i);
    }

    return -1;
}


///////
//
//  setFlag
//
//     set a flag in the info struct
//     Search directory [dirName] looking for [fileName]. Matches are
//     printed using printf.  When a directory is found searchdir calls
//     itself passing that directory as the dirName. 
//
//     args: *info     --  path of directory to search.
//            item    --  name of the file being looked for
//            switchIs
//
void setFlag(struct termios *info, int item, int switchIs)
{
    if (switchIs) {
        
        // Switch OFF
        switch (mFlagArgs[item].flagType) {
            case INPUTFLAG:
                info->c_iflag &= ~mFlagArgs[item].theValue;
                break;
            case OUTPUTFLAG:
                info->c_oflag &= ~mFlagArgs[item].theValue;
                break;
            case CONTROLFLAG:
                info->c_cflag &= ~mFlagArgs[item].theValue;
                break;
            case LOCALFLAG:
                info->c_lflag &= ~mFlagArgs[item].theValue;
                break;
        }
    }
    else {
        
        // Switch ON
        switch (mFlagArgs[item].flagType) {
            case INPUTFLAG:
                info->c_iflag |= mFlagArgs[item].theValue;
                break;
            case OUTPUTFLAG:
                info->c_oflag |= mFlagArgs[item].theValue;
                break;
            case CONTROLFLAG:
                info->c_cflag |= mFlagArgs[item].theValue;
                break;
            case LOCALFLAG:
                info->c_lflag |= mFlagArgs[item].theValue;
                break;
            }
    }
}

    
///////
//
//  getAscii
//
//     convert standard ascii chars to control char ascii
//     Search directory [dirName] looking for [fileName]. Matches are
//     printed using printf.  When a directory is found searchdir calls
//     itself passing that directory as the dirName. 
//
//     args: *str     --  path of directory to search.
//
int getAscii(char *str)
{   
    int ascii = 0;
    
    if ( strlen(str) == 1 )                    // arg length is 1
        ascii = str[0];                        // ...get ascii from 1st char
                
    else if ( strlen(str) == 2
                && str[0] == '^' ) {           // arg length is 2 and '^'
                
        ascii = str[1];                        // ...get ascii from 2nd char
                
        if (ascii == '?')                      // convert "^?" to backspace
            ascii = 127;
        
        if (ascii > 96) 
            ascii = ascii - 'a' + 1;           // lower case to cntrl ascii
        else
            ascii = ascii - 'A' + 1;           // upper case to cntrl ascii

        if (ascii < 0 || ascii > 31)           // keep within ctrl limit
            ascii = 0;                        // BAD VALUE
    }
    
    return ascii;
}


///////
//
//  displaySettings
//
//     Search directory [dirName] looking for [fileName]. Matches are
//     printed using printf.  When a directory is found searchdir calls
//     itself passing that directory as the dirName. 
//
//     args: *info     --  path of directory to search.
//
void displaySettings(struct termios *info)
{    
    int i;  
        
    showbaud( cfgetospeed( info ));  // get and show baud rate
    printf("\n");

    for ( i = 0; mKeyArgs[i].theValue ; i++ ) {
        printf("%s = ", mKeyArgs[i].attribute);
        printASCII(info->c_cc[mKeyArgs[i].theValue]);
    }
    printf("\n");
   
    // test for flag attribute
    for ( i = 0; mFlagArgs[i].theValue ; i++ ) {
        printFlag( info, mFlagArgs[i].attribute,
                         mFlagArgs[i].theValue,
                         mFlagArgs[i].flagType);
    }
    printf("\n"); 
}


///////
//
//  showbaud
//
//     prints the speed in english
//     Search directory [dirName] looking for [fileName]. Matches are
//     printed using printf.  When a directory is found searchdir calls
//     itself passing that directory as the dirName. 
//
//     args:  thespeed     --  path of directory to search.
//
void showbaud( int thespeed )
{
        printf("speed ");
        switch ( thespeed ){
                case B300:      printf("300");        break;
                case B600:      printf("600");        break;
                case B1200:     printf("1200");       break;
                case B1800:     printf("1800");       break;
                case B2400:     printf("2400");       break;
                case B4800:     printf("4800");       break;
                case B9600:     printf("9600");       break;
                default:        printf("Fast");       break;
        }
        printf(" baud; ");
}


///////
//
//  printFlag
//
//     Search directory [dirName] looking for [fileName]. Matches are
//     printed using printf.  When a directory is found searchdir calls
//     itself passing that directory as the dirName. 
//
//     args: *ttyp         --  path of directory to search.
//           *flagName    --  name of the file being looked for
//            theFlag    --  name of the file being looked for
//            flagType    --  name of the file being looked for
//
void printFlag( struct termios *ttyp,
                char *flagName,
                int theFlag, 
                int flagType)
{
    int theValue = 0;

    switch (flagType) {
        case INPUTFLAG:
            theValue = ttyp->c_iflag;
            break;
        case OUTPUTFLAG:
            theValue = ttyp->c_oflag;
            break;
        case CONTROLFLAG:
            theValue = ttyp->c_cflag;
            break;
        case LOCALFLAG:
            theValue = ttyp->c_lflag;
            break;
    }

    if (theValue & theFlag)
            printf("%s ", flagName);
    else
            printf("-%s ", flagName);
}


///////
//
//  printASCII
//
//     Given an ASCII code print the readible symbol
//     Search directory [dirName] looking for [fileName]. Matches are
//     printed using printf.  When a directory is found searchdir calls
//     itself passing that directory as the dirName. 
//
//     args:  ascii     --  path of directory to search.
//
void printASCII(int ascii)
{
    if (ascii == 127)
        printf("^?; ");
    else if (ascii > 0 || ascii < 32)
        printf("^%c; ", ascii -1 + 'A');
    else
        printf("%c; ", ascii);    
}




/////////
//
/* All the flags in the termios.h file
//  formatted for
// 
//  struct flaginfo { int   fl_value; char  *fl_name; };

//
// Input flags - software input processing
//
struct flaginfo inputFlags[] = {
                  {IGNBRK    , "ignore BREAK condition"},
                  {BRKINT    , "map BREAK to SIGINTR"},
                  {IGNPAR    , "ignore (discard) parity errors"},
                  {PARMRK    , "mark parity and framing errors"},
                  {INPCK     , "disable checking of parity errors"},
                  {ISTRIP    , "strip 8th bit off chars"},
                  {INLCR     , "map NL into CR"},
                  {IGNCR     , "ignore CR"},
                  {ICRNL     , "map CR to NL (ala CRMOD)"},
                  {IXON      , "enable output flow control"},
                  {IXOFF     , "enable input flow control"},
                  {IXANY     , "any char will restart after stop"},
                  {IUCLC     , "DUMMY VALUE Map upper to lower case on input"},
                  {IFLOW     , "enable output flow control"},
                  {ITANDEM   , "enable input flow control"},
                  {IMAXBEL   , "ring bell on input queue full"},
                  {0         ,  NULL }};
//
// Output flags - software output processing
//
struct flaginfo outputFlags[] = {
                     {OPOST     , "enable following output processing"},
                     {ONLCR     , "map NL to CR-NL (ala CRMOD)"},
                     {OLCUC     , "Map lower case to upper on output"},
                     {OCRNL     , "Map CR to NL on output"},
                     {ONOCR     , "No CR output at column 0"},
                     {ONLRET    , "NL performs CR function"},
                     {OFILL     , "Use fill characters for delay"},
                     {OFDEL     , "fill is DEL, else NUL"},
                     {NLDLY     , "\\n delay"},
                     {NL0       , "NL0"},
                     {NL1       , "NL1 tty 37"},
                     {NL2       , "NL2 vt05"},
                     {NL3       , "NL3"},
                     {TABDLY    , "horizontal tab delay"},
                     {TAB0      , "TAB0"},
                     {TAB1      , "TAB1 tty 37"},
                     {TAB2      , "TAB2"},
                     {TAB3      , "TAB3 expand tabs on output"},
                     {CRDLY     , "\\r delay"},
                     {CR0       , "CR0"},
                     {CR1       , "CR1 tn 300"},
                     {CR2       , "CR2 tty 37"},
                     {CR3       , "CR3 concept 100"},
                     {FFDLY     , "Form feed delay"},
                     {FF0       , "FF0"},
                     {FF1       , "FF1"},
                     {BSDLY     , "\\b delay"},
                     {BS0       , "BS0"},
                     {BS1       , "BS1"},
                     {VTDLY     , "vertical tab delay"},
                     {VT0       , "VT0"},
                     {VT1       , "VT1 tty 37"},
                     {ONLCRNL   , "ONLCRNL"},
                     {OXTABS    , "expand tabs to spaces"},
                     {ONOEOT    , "discard EOT's (^D) on output)"},
                     {0         ,  NULL }};                  
//
// Control flags - hardware control of terminal
//
struct flaginfo cntrFlags[] = {
                     {CSIZE     , "character size mask"},
                     {CSTOPB    , "send 2 stop bits"},
                     {CREAD     , "enable receiver"},
                     {PARENB    , "parity enable"},
                     {PARODD    , "odd parity, else even"},
                     {HUPCL     , "hang up on last close"},
                     {CLOCAL    , "ignore modem status lines"},
                     {CRTSCTS   , "RTS/CTS flow control"},
                     {0         ,  NULL }};

                     //{CS5       , "5 bits (pseudo)"},
                     //{CS6       , "6 bits"},
                     //{CS7       , "7 bits"},
                     //{CS8       , "8 bits"},

// 
// "Local" flags - dumping ground for other state
//
// Warning: some flags in this structure begin with
// the letter "I" and look like they belong in the
// input flag.
//
struct flaginfo localFlags[] = {
                     {ECHOE     , "visually erase chars"},
                     {ECHOK     , "echo NL after line kill"},
                     {ECHO      , "enable echoing"},
                     {ECHONL    , "echo NL even if ECHO is off"},
                     {ISIG      , "enable signals INTR, QUIT, [D]SUSP"},
                     {ICANON    , "canonicalize input lines"},
                     {IEXTEN    , "enable FLUSHO and LNEXT"},
                     {NOFLSH    , "don't flush after interrupt"},
                     {TOSTOP    , "stop background jobs from output"},
                     {XCASE     , "Cononical upper/lower presentation"},
                     {ECHOKE    , "visual erase for line kill"},
                     {ECHOPRT   , "visual erase mode for hardcopy"},
                     {ECHOCTL   , "echo control chars as ^(Char)"},
                     {ALTWERASE , "use alternate WERASE algorithm"},
                     {MDMBUF    , "flow control output via Carrier"},
                     {FLUSHO    , "output being flushed (state)"},
                     {NOHANG    , "this should go away"},
                     {PENDIN    , "retype pending input (state)"},
                     {NOKERNINFO, "Disable printing kernel info"},
                     {0         , NULL }};

*/
